@@ -59,6 +59,7 @@ import System.Posix.Signals     (raiseSignal, sigTSTP, installHandler, Handler(.
 
 import Foreign.C.String
 import Foreign.C.Types
+import Foreign.C.Error (Errno(..), getErrno)
 
 import qualified Data.ByteString.Char8 as P
 import qualified Data.ByteString as B
@@ -132,20 +133,35 @@ suspend = raiseSignal sigTSTP
 screenSize :: IO (Int, Int)
 screenSize = Curses.scrSize
 
+--
+-- | Rewrite of Curses.getCh to avoid looping on terminal crash
+-- | (also no unget support since I don't need it)
+--
+getCh :: IO Curses.Key
+getCh = do
+  threadWaitRead 0
+  v <- Curses.getch
+  case v of
+    -1 -> do
+        Errno e <- getErrno
+        putStrLn $ "Error " ++ show e ++ "; terminal has gone away?  Hard-exiting now."
+        exitFailure
+    k -> pure $ Curses.decodeKey k
+
 --
 -- | Read a key. UIs need to define a method for getting events.
 -- We only need to refresh if we don't have no SIGWINCH support.
 --
 getKey :: IO Char
 getKey = do
-    k <- Curses.getCh
+    k <- getCh
     if k == Curses.KeyResize 
         then do
               when (isNothing Curses.cursesSigWinch) do
                   runDraw $ redraw <> resizeui
               getKey
         else pure $ unkey k
- 
+
 -- | Resize the window
 -- From "Writing Programs with NCURSES", by Eric S. Raymond and Zeyd M. Ben-Halim
 --
@@ -332,7 +348,7 @@ instance ModalElement HistModal where
         let wd = modalWidth swd
             mtlen = maximum $ map (length . fst) hist
             tlen = min (mtlen + 1) $ wd `div` 3
-        (wd,) $ flip map (zip (['0'..'9']++['a'..'z']) hist) \ (c, (time, song)) ->
+        (wd,) $ flip map (zip (['0'..'9']++['a'..'z']) hist) \ (c, (time, (_, song))) ->
             let tstr = ellipsize tlen $ replicate (tlen - displayWidth time) ' ' ++ time
             in Fast (UTF8.fromString $ forceWidth wd $ ' ' : c : ' ' : tstr ++ ' ' : song) sty
 
@@ -406,7 +422,7 @@ instance Element PTime where
 instance Element PMode where
     draw dd = PMode case status $ drawState dd of
                         Stopped -> "◼"
-                        Paused  -> "Ⅱ"
+                        Paused  -> "⏸"
                         Playing -> "▶"
 
 -- | Loop, normal, or random
@@ -414,7 +430,8 @@ instance Element PMode2 where
     draw dd = PMode2 case mode $ drawState dd of
                         Random  -> "rand"
                         Loop    -> "loop"
-                        Normal  -> "once"
+                        Once    -> "once"
+                        Single  -> "sing"
 
 ------------------------------------------------------------------------
 
